// -*- C++ -*-
/*!
* @file  LeapMotionData.cpp
* @brief LeapMotionData Output
* @date $Date$
*
* $Id$
*/



#include <LeapMotionData.h>
Hand now;
int i=0;

// Module specification
// <rtc-template block="module_spec">
static const char* leapmotiondata_spec[] =
{
	"implementation_id", "LeapMotionData",
	"type_name",         "LeapMotionData",
	"description",       "LeapMotionData Output",
	"version",           "1.0.0",
	"vendor",            "AIST",
	"category",          "Category",
	"activity_type",     "PERIODIC",
	"kind",              "DataFlowComponent",
	"max_instance",      "1",
	"language",          "C++",
	"lang_type",         "compile",
	""
};
// </rtc-template>

/*!
* @brief constructor
* @param manager Maneger Object
*/
LeapMotionData::LeapMotionData(RTC::Manager* manager)
	// <rtc-template block="initializer">
	: RTC::DataFlowComponentBase(manager),
	m_CalibrationOut("Calibration_OP", m_Calibration),
	m_Hand_PosOut("Hand_Position_OP", m_Hand_Pos),
	m_Hand_AngleOut("Hand_Angle_OP", m_Hand_Angle),
	m_Hand_OpenOut("Hand_Open_OP", m_Hand_Open)

	// </rtc-template>
{
}

/*!
* @brief destructor
*/
LeapMotionData::~LeapMotionData()
{
}



RTC::ReturnCode_t LeapMotionData::onInitialize()
{

	// Registration: InPort/OutPort/Service
	// <rtc-template block="registration">
	// Set InPort buffers

	// Set OutPort buffer
	addOutPort("Calibration_OP", m_CalibrationOut);
	addOutPort("Hand_Position_OP", m_Hand_PosOut);
	addOutPort("Hand_Angle_OP", m_Hand_AngleOut);
	addOutPort("Hand_Open_OP", m_Hand_OpenOut);

	// Set service provider to Ports

	// Set service consumers to Ports

	// Set CORBA Service Ports

	// </rtc-template>

	// <rtc-template block="bind_config">
	// </rtc-template>

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t LeapMotionData::onFinalize()
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LeapMotionData::onStartup(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LeapMotionData::onShutdown(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t LeapMotionData::onActivated(RTC::UniqueId ec_id)
{
	m_pLeapListener = new SampleListener();
	m_pLeapController = new Leap::Controller();
	m_pLeapController->addListener(*m_pLeapListener);


	return RTC::RTC_OK;
}


RTC::ReturnCode_t LeapMotionData::onDeactivated(RTC::UniqueId ec_id)
{
	m_pLeapController->removeListener(*m_pLeapListener);
	delete m_pLeapController;
	delete m_pLeapListener;

	return RTC::RTC_OK;
}


RTC::ReturnCode_t LeapMotionData::onExecute(RTC::UniqueId ec_id)
{


	Leap::Frame &frame = m_pLeapListener->m_Frame;

	HandList hands = frame.hands();
	const GestureList gestures = frame.gestures();
	for(int g=0 ;g < gestures.count(); ++g){
		if(g >= 1) break;
		//i++;
		Gesture gesture = gestures[g];
		switch(gesture.type()){
		case Gesture::TYPE_KEY_TAP:
			{
				KeyTapGesture tap = gesture;
				if(tap.position().y > m_Calibration.data.y){
				/*std::cout << std::string(2, ' ')*/
				//<< "Key Tap id: " << gesture.id()
				////<< ", state: " << stateNames[gesture.state()]
				//<< ", position: " << tap.position()
				//<< ", direction: " << tap.direction()<< std::endl;
				m_Calibration.data.x = tap.position().x;
				m_Calibration.data.y = tap.position().y;
				m_Calibration.data.z = tap.position().z;
				m_CalibrationOut.write();
				break;
				}
			}
		}
	}

	for (HandList::const_iterator hl = hands.begin(); hl != hands.end(); ++hl) {
		// Get the first hand
		const Hand hand = *hl;
		const Vector normal = hand.palmNormal();
		const Vector direction = hand.direction();
		//std::string handType = hand.isLeft() ? "Left hand" : "Right hand";
		//std::cout << std::string(2, ' ') << handType << ", id: " << hand.id()
		//	<< ", palm position: " << hand.palmPosition() << std::endl;

		m_Hand_Pos.data.x = hand.palmPosition().x;
		m_Hand_Pos.data.y = hand.palmPosition().y;
		m_Hand_Pos.data.z = hand.palmPosition().z;
		m_Hand_Angle.data.x = normal.roll();
		m_Hand_Angle.data.y = direction.pitch();
		m_Hand_Angle.data.z = direction.yaw();
		m_Hand_Open.data = hand.grabStrength();

		
		//std::cout << "x = " << m_Hand_Pos.data.x << ",y = " << m_Hand_Pos.data.y << " ,z = " << m_Hand_Pos.data.z << std::endl;
		//std::cout << "roll = " << m_Hand_Angle.data.x << ",pitch = " << m_Hand_Angle.data.y << " ,yaw = " << m_Hand_Angle.data.z << std::endl;

		m_Hand_PosOut.write();
		m_Hand_AngleOut.write();
		m_Hand_OpenOut.write();
	}

	return RTC::RTC_OK;

}

/*
RTC::ReturnCode_t LeapMotionData::onAborting(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LeapMotionData::onError(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LeapMotionData::onReset(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LeapMotionData::onStateUpdate(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LeapMotionData::onRateChanged(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

extern "C"
{

	void LeapMotionDataInit(RTC::Manager* manager)
	{
		coil::Properties profile(leapmotiondata_spec);
		manager->registerFactory(profile,
			RTC::Create<LeapMotionData>,
			RTC::Delete<LeapMotionData>);
	}

};
